var _TextInput, _TextInput2, _TextInput3, _TextInput4, _TextInput5, _TextInput6, _TextInput7;
/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2015 - present Instructure, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import { render, screen, waitFor, fireEvent } from '@testing-library/react';
import userEvent from '@testing-library/user-event';
import { vi } from 'vitest';
import '@testing-library/jest-dom';
import { runAxeCheck } from '@instructure/ui-axe-check';
import { TextInput } from '../index';
import { jsx as _jsx } from "@emotion/react/jsx-runtime";
describe('<TextInput/>', () => {
  let consoleErrorMock;
  beforeEach(() => {
    // Mocking console to prevent test output pollution and expect for messages
    consoleErrorMock = vi.spyOn(console, 'error').mockImplementation(() => {});
  });
  afterEach(() => {
    consoleErrorMock.mockRestore();
  });
  it('should include a label', async () => {
    const _render = render(_TextInput || (_TextInput = _jsx(TextInput, {
        renderLabel: "Name"
      }))),
      container = _render.container;
    const label = container.querySelector('label');
    expect(label).toHaveTextContent('Name');
  });
  it('should focus the input when focus is called', async () => {
    var _ref;
    let ref;
    render(_jsx(TextInput, {
      renderLabel: "Name"
      //@ts-expect-error TODO this is coming from ReactComponentWrapper
      ,
      inputRef: el => {
        ref = el;
      }
    }));
    const input = screen.getByRole('textbox');
    (_ref = ref) === null || _ref === void 0 ? void 0 : _ref.focus();
    expect(input).toHaveFocus();
  });
  it('should provide an inputRef prop', async () => {
    const inputRef = vi.fn();
    render(_jsx(TextInput, {
      renderLabel: "Name",
      inputRef: inputRef
    }));
    const input = screen.getByRole('textbox');
    expect(inputRef).toHaveBeenCalledWith(input);
  });
  it('should provide a value getter', async () => {
    var _ref2;
    let ref;
    render(_jsx(TextInput, {
      renderLabel: "Name",
      defaultValue: "bar"
      //@ts-expect-error TODO this is coming from ReactComponentWrapper
      ,
      inputRef: el => {
        ref = el;
      }
    }));
    expect((_ref2 = ref) === null || _ref2 === void 0 ? void 0 : _ref2.value).toBe('bar');
  });
  it('should let aria-describedby through', async () => {
    render(_TextInput2 || (_TextInput2 = _jsx(TextInput, {
      renderLabel: "Name",
      "aria-describedby": "abcd"
    })));
    const input = screen.getByRole('textbox');
    expect(input).toHaveAttribute('aria-describedby', 'abcd');
  });
  describe('events', () => {
    it('responds to onChange event', async () => {
      const onChange = vi.fn();
      render(_jsx(TextInput, {
        renderLabel: "Name",
        onChange: onChange
      }));
      const input = screen.getByRole('textbox');
      fireEvent.change(input, {
        target: {
          value: 'foo'
        }
      });
      await waitFor(() => {
        expect(onChange).toHaveBeenCalledTimes(1);
      });
    });
    it('responds to onBlur event', async () => {
      const onBlur = vi.fn();
      render(_jsx(TextInput, {
        renderLabel: "Name",
        onBlur: onBlur
      }));
      userEvent.tab();
      userEvent.tab();
      await waitFor(() => {
        expect(onBlur).toHaveBeenCalled();
      });
    });
    it('responds to onFocus event', async () => {
      const onFocus = vi.fn();
      render(_jsx(TextInput, {
        renderLabel: "Name",
        onFocus: onFocus
      }));
      const input = screen.getByRole('textbox');
      input.focus();
      await waitFor(() => {
        expect(onFocus).toHaveBeenCalled();
      });
    });
  });
  describe('interaction', () => {
    it('should set the disabled attribute when `interaction` is disabled', async () => {
      render(_TextInput3 || (_TextInput3 = _jsx(TextInput, {
        renderLabel: "Name",
        interaction: "disabled"
      })));
      const input = screen.getByRole('textbox');
      expect(input).toHaveAttribute('disabled');
    });
    it('should set the disabled attribute when `disabled` is set', async () => {
      render(_TextInput4 || (_TextInput4 = _jsx(TextInput, {
        renderLabel: "Name",
        disabled: true
      })));
      const input = screen.getByRole('textbox');
      expect(input).toHaveAttribute('disabled');
    });
    it('should set the readonly attribute when `interaction` is readonly', async () => {
      render(_TextInput5 || (_TextInput5 = _jsx(TextInput, {
        renderLabel: "Name",
        interaction: "readonly"
      })));
      const input = screen.getByRole('textbox');
      expect(input).toHaveAttribute('readonly');
    });
    it('should set the readonly attribute when `readOnly` is set', async () => {
      render(_TextInput6 || (_TextInput6 = _jsx(TextInput, {
        renderLabel: "Name",
        readOnly: true
      })));
      const input = screen.getByRole('textbox');
      expect(input).toHaveAttribute('readonly');
    });
  });
  describe('for a11y', () => {
    it('should meet standards', async () => {
      const _render2 = render(_TextInput7 || (_TextInput7 = _jsx(TextInput, {
          renderLabel: "Name"
        }))),
        container = _render2.container;
      const axeCheck = await runAxeCheck(container);
      expect(axeCheck).toBe(true);
    });
    it('should set aria-invalid when errors prop is set', async () => {
      render(_jsx(TextInput, {
        renderLabel: "Name",
        messages: [{
          type: 'error',
          text: 'some error message'
        }]
      }));
      const input = screen.getByRole('textbox');
      expect(input).toHaveAttribute('aria-invalid');
    });
  });
});